/***************************************************************************
 *   Copyright (C) 2004-2008 by Pere Constans
 *   constans@molspaces.com
 *   cb2Bib version 0.9.4
 *   See the LICENSE file that comes with this distribution
 ***************************************************************************/
#ifndef C2BBIBPARSER_H
#define C2BBIBPARSER_H

#include "bibParser.h"
#include "c2bLineEdit.h"


class QComboBox;
class QWidget;
class c2bBibPreparser;
class c2bCiteIDMaker;
class c2bClipEdit;
class c2bPreprocess;
class c2bSettings;
class cb2bibbase;
class journalDB;


/*!
    cb2Bib interface to bibParser

    This class groups Bib Parser functionality related to GUI

    @author Pere Constans
*/
class c2bBibParser : public bibParser
{

    Q_OBJECT

public:
    c2bBibParser(QObject *parent = 0);
    ~c2bBibParser();

    QHash<QString, c2bLineEdit*> bibFields;
    QString abbreviatedJournal(const QString& j);
    QString c2bAutoRecogStr;
    QString excerpt(const QString& text, const QStringList& hints);
    QString fullJournal(const QString& j);
    QString setJournalsToAbbreviated(const QString& text);
    QString setJournalsToFull(const QString& text);
    bool m_auto_regc;
    void clearFields();
    void connectBibWidgets(QHash<QString, c2bLineEdit*>& fields, QComboBox* types, c2bClipEdit* editor);
    void guessItem(const QString& tagged_text);
    void makeCiteID();

signals:
    void bibModified();
    void preParserDataAvailable(const QString& ppItem);


public slots:
    QString PProc(const QString, QString);
    QString makeBib(bool sig = true);
    void preParseItem(const QString& cb);
    void processItem(QString);
    void setJournal();


protected:
    journalDB *JDB;


private:
    QComboBox* bibTypes;
    QString removeTags(const QString& text);
    QString setTags(const QString& text);
    QWidget* c2bMain;
    c2bBibPreparser *prParser;
    c2bCiteIDMaker *citeIDM;
    c2bClipEdit* bibEditor;
    c2bPreprocess *prProc;
    c2bSettings *settings;
    int countFields();
    void guessAbstract(const QString& text);
    void guessISBN(const QString& text);
    void guessJournal(QString text);
    void guessKeywords(const QString& text);
    void guessNumber(const QString& text);
    void guessPages(const QString& text);
    void guessTitle(const QString& text);
    void guessVolume(const QString& text);
    void guessYear(const QString& text);
    void setRefType(QString);

};

#endif
