/***************************************************************************
 *   Copyright (C) 2004-2008 by Pere Constans
 *   constans@molspaces.com
 *   cb2Bib version 0.9.4
 *   See the LICENSE file that comes with this distribution
 ***************************************************************************/
#ifndef C2BBIBSEARCHER_H
#define C2BBIBSEARCHER_H

#include "c2bBibSearcherCache.h"

#include <QMap>
#include <QObject>
#include <QStringList>


/***************************************************************************
 *   class c2bSearchPattern
 ***************************************************************************/

class c2bSearchPattern
{

public:
    c2bSearchPattern(bool newNot, bool newCaseSensitive, const QString& newPatternType, const QString& newScope,
                     const QString& newYearScope, const QString& newPattern);

    bool Not;
    QRegExp rx;
    QString pattern;
    QString scope;
    QString yearScope;

    inline const QString toString() const
    {
        return _pStr;
    }

    inline bool operator< (const c2bSearchPattern& p) const
    {
        return (_rank < p._rank);
    }


private:
    QString _pStr;
    int _rank;

};


/***************************************************************************
 *   class c2bBibSearcher
 ***************************************************************************/

class bibReference;
class c2bBibParser;

class c2bBibSearcher : public QObject
{

    Q_OBJECT

public:
    c2bBibSearcher(QObject *parent = 0);
    ~c2bBibSearcher();

    const QString highlight(const QString& abstract);
    int errorCount();
    int hitCount();
    int patternCount();
    int referenceCount();
    QMap<QString, QString> hitMap;
    QString hitString;
    QString logString;
    void addPattern(bool Not, bool caseSensitive, const QString& patternType, const QString& scope,
                    const QString& yearScope, const QString& pattern);
    void exec();
    void setBoolean(bool And);
    void setScope(const QString& file, const QString& dir, bool all, bool includePdf);
    void setSimplifySource(bool simplify);


public slots:
    void abort();


private:
    bool p_aborted;
    bool p_all_bibtex_files;
    bool p_boolean_and;
    bool p_include_pdf;
    bool p_simplify_source;
    c2bBibParser* bp;
    c2bBibSearcherCache cache;
    const QString excerpts(const QString& contents);
    int bibtexCounter;
    int errorCounter;
    int pdfCounter;
    int referenceCounter;
    QList<c2bSearchPattern> patternList;
    QStringList fieldList;
    QString p_bibtex_dir;
    QString p_bibtex_file;
    void clear();
    void searchBib(const bibReference& ref);
    void search(const QString& bib_file);

    inline bool include_pdf(bool include, const QString& file_name, QString* file_contents)
    {
        if (!include)
            return false;
        if (*file_contents == "unread")
            *file_contents = cache.pdfToText(file_name, &pdfCounter, &logString, &errorCounter);
        return !(file_contents->isEmpty());
    }

};

#endif
