/***************************************************************************
 *   Copyright (C) 2004-2008 by Pere Constans
 *   constans@molspaces.com
 *   cb2Bib version 0.9.4
 *   See the LICENSE file that comes with this distribution
 ***************************************************************************/
#include "c2bConfigure.h"
#include "c2b.h"
#include "c2bConfigure_FR.h"
#include "c2bFileDialog.h"
#include "c2bSettings.h"

#include <QColorDialog>
#include <QHeaderView>


/*! \page c2bconf Configuring cb2Bib

- \ref c2bconf_files
- \ref c2bconf_fonts
- \ref c2bconf_input
- \ref c2bconf_network
- \ref c2bconf_output
- \ref c2bconf_pdf2text
<p>&nbsp;</p>


\section c2bconf_files Configuring Files

    - <b>Journal Abbreviation List File</b> \n This file contains a list of
    journal names equivalences: a capital-letter acronym, standard abbreviated
    form, and full name of the journal. If an input journal name is recognized,
    cb2Bib will use the standard abbreviated form for the <tt>journal</tt>
    bibkey. If your usual journal were not within the distributed, default
    <tt>abbreviations.txt</tt>, you could edit this file, or point to a
    personalized abbreviation file. <b>Note:</b> Changes in the abbreviation
    file only take place after reestarting cb2Bib.

    - <b>Regular Expression List File</b> \n The cb2Bib distribution includes
    the file <tt>regexps.txt</tt> with a few set of rules for reference
    extraction. This includes most of the scientific literature. Extracting
    from email alerts or publisher abstract pages is a <em>volatile</em> task.
    Information does not follow a standardized structure. Extraction pattern
    may then change as often as the web design needs to. Besides, such
    extraction from the clipboard is system dependent, in a way that produces
    different formatting of the text copies on different systems. You can use
    your personalized <tt>regexps.txt</tt> file, for testing, debugging
    -regular expressions are reloaded each time the automatic recognition
    engine executes-, and fullfiling your particular extraction needs.

    - <b>Bookmarks and Network Query Info File</b> \n The cb2Bib distribution
    includes the file <tt>netqinf.txt</tt> that contains bookmarks data, and
    server related information for bibliographic querying. Note that the cb2Bib
    treats bibliographic queries as generalized net bookmarks. This allows
    accessing almost any online bibliographic resource. Check this file for
    implementations details and customization.

    - <b>Browser Cascading Style Sheet (CSS)</b> \n This file contains the
    style sheet to configure the appearance of the bibliographic references
    when viewed in browser mode. The cb2Bib distribution includes the
    <tt>references.css</tt> file as a CSS example.

    - <b>Search In Files Cache Directory</b> \n Directory containing internal
    data for Search In Files functionality. If an existing directory is
    selected the cb2Bib will write all internal data on it. If otherwise, cache
    data will be written on the same directory from where BibTeX are searched.
    It might be, therefore, convenient to group all this files in a separate
    directory that does not need to be backup, and that can easily be deleted
    whenever desired.

    - <b>Rename and Copy/Move article files to Directory</b> \n If selected,
    each file 'drag and dropped' onto the cb2Bib main window is renamed to
    <tt>citeID.pdf</tt> (or citeID.ps, citeID.dvi, etc.) and moved to the
    pointed storage directory. If unselected, the file URL is written to the
    <tt>file</tt> bibkeyword, without any renaming or moving of the file. The
    actual copy/move action is scheduled and performed once the reference is
    accepted, e.g., once it is saved.

    - <b>Copy or Move article files</b> \n Choose whether copy or move Network
    Files 'dropped' onto the cb2Bib main window. See also <b>Use External
    Network Client</b>.

    - <b>Set Article Files Directory Relative to current BibTeX file
    Directory</b> \n If checked, the article file is copied/moved to the
    current BibTeX file directory. Only the filename, instead of the fullpath
    filename, is written to the BibTeX entry. File retrieving from within the
    cb2Bib browser will be relative to the BibTeX file absolute location. Use
    this option if you plan to store in a same directory the BibTeX and article
    files. This option is appropriate for storing bibliographic collections in
    removal devices.


\section c2bconf_fonts Configuring Fonts

    - <b>Font Selector</b> \n Selects the main window and editor font family
    and size.

    - <b>Context Colors</b> \n Doubleclick on context color items to select
    syntax highlighter font color. Besides syntax highlighting, and to ease
    manual bibliographic extractions, cb2Bib has the following coloring
    convention. 'cb2Bib unrelevant text' colors non-word, non-digit, and
    cb2Bib's internal tags. 'cb2Bib relevant text' refers to the reference's
    year. 'cb2Bib highly relevant' attempts to guess text sectioning,
    highlighting 'abstract', 'introduction', and 'keywords'.


\section c2bconf_input Configuring Input

    - <b>Replace/Remove from input stream</b> \n If checked, input stream is
    preprocessed by performing a customizable set of string
    substitutions/removals. This option is mainly intended to remove image HTML
    <tt>alt</tt> tags. Although not visible, <tt>alt</tt> tags reach the
    clipboard when selecting and copying text. Author lists embellished with
    email icons may therefore contain <tt>alt</tt> strings that would interfere
    with the author names processing. In addition, this option is also
    appropriate to help translating special characters to Unicode and LaTeX.
    Use it carefully, as to avoid unwanted or unsuspected substitutions. See
    also \ref bibproc.

    - <b>External Clipboard Preparsing Command</b> \n Preparses input stream
    through an external, user-defined tool. Use the box bellow to specify its
    name and path. cb2Bib executes the command <tt>tool_name tmp_inputfile
    tmp_outputfile</tt>. You might consider a wrapper shell script to fullfill
    this particular syntax requirement. Two examples, <tt>isi2bib</tt> and
    <tt>ris2bib</tt> are provided. To test them, make sure the
    \htmlonly
    <a href="http://www.scripps.edu/~cdputnam/software/bibutils/bibutils.html"
    target="_blank">BibUtils Package</a>
    \endhtmlonly
    is available on your machine. Otherwise, modify these scripts according to
    your needs. See also \ref keyboard, \ref bibproc, and the examples \ref
    ex_isi_format and \ref ex_ris.

    - <b>Perform always, as part of an automatic extraction</b> \n Performs
    preparsing each time the recognition engine is invoked. <b>Caution:</b>
    cb2Bib, when not in disconnected mode, starts the recognition engine each
    time the clipboard changes. Thus, it might send arbitrary data to the
    external parsing tool. The tool might not be prepared to handle <b>'any
    data'</b> and might fall into a sort of <b>'infinite loop'</b>. cb2Bib
    kills the external tool after a reasonable waiting. But, if the tool is
    called through a wrapper script, killing the script will not end the tool
    itself. Therefore, <b>check this box only when needed</b>. If you write
    your own preparser, design it as to write no data to output file whenever
    it can not process an input properly. When the preparser produces no data,
    the cb2Bib sends instead the input stream to the recognition engine. In
    this way, preparsing and normal cb2Bib functioning will work harmoniously.

    - <b>Do not show log output</b> \n If unchecked, the external process
    messages, and the input and output streams are shown in a log window.
    Showing output logs is usefull for debugging purposes.


\section c2bconf_network Configuring Network

    - <b>Use External Network Client</b> \n cb2Bib manages local and network
    files in an equivalent manner. Network file retrieving, however, requires
    sometimes password and/or cookies administration. Opening and browsing
    retrieved files requires also desktop specific commands. The KDE desktop
    incorporates <tt>kfmclient</tt> utility. A command <tt>kfmclient
    (copy|move|exec) orig [dest]</tt> permits copying, moving or opening files,
    taking care of redirections, cookies or user browsing preferences. By
    checking this box, cb2Bib will use the specified file manger client.
    External file managers should be used whenever redirections or cookies are
    an issue. <b>Note:</b> Internal Network file manager has been found to
    'freeze' on some Windows platforms. Switch to an appropriate external
    manager if experiencing problems.

    - <b>Perform Network Queries after automatic reference extractions</b> \n
    Network queries can be used to complete a partial reference extraction. For
    instance, provided a reference 'J. Name, 25, 103' and an appropriate
    pattern to extract it, cb2Bib will attempt to complete the reference
    automatically. Currently, it should work out-of-the-box for PubMed, PNAS,
    and ACS publications. See also the file <tt>netqinf.txt</tt>.

    - <b>Download PDF article if available</b> \n If checked, cb2Bib downloads
    article PDF files to the directory specified in <b>Rename and Copy/Move
    dropped PDF article files to Directory</b>. See also the file
    <tt>netqinf.txt</tt> for details. Download is scheduled and performed once
    the reference is accepted, e.g., once it is saved.

    - <b>Keep Query temporary files (Debug Only)</b> \n cb2Bib may use up to
    three temporary files to perform a network query. If this box is checked,
    the temporary files are not deleted. This facilitates the customization of
    the information file <tt>netqinf.txt</tt>.


\section c2bconf_output Configuring Ouput

    - <b>Author and Editor Name Format</b> \n Sets Authors and Editor names in
    abbreviated or full form, if the latter is available.

    - <b>Journal Name Format</b> \n Sets output journal names in abbreviated or
    full form. Note that this feature only works if journal names are found in
    the <tt>Journal Abbreviation List File</tt>.

    - <b>CiteID Pattern</b> \n Specifies the pattern for formating cite's ID.
    Pattern editing and predefined placeholders popup menu are available upon
    mouse rightclicks. Placeholders will be substituted by the actual cite
    field values. See \ref citeid_placeholders for descriptions.

    - <b>Convert entry strings to LaTeX</b> \n If checked, cb2Bib converts
    special, Unicode characters to LaTeX commands. Most BibTeX import filters
    do not process LaTeX escaped characters. Keeping this box unchecked can
    thus be appropriate when using cb2Bib-BibTeX as a temporary media to
    transfer references to non BibTeX reference managers and formats.

    - <b>Set 'title' in double braces</b> \n If checked, it writes extra braces
    in title. This will keep capitalization as is, when processed by BibTeX.

    - <b>Postprocess 'month'</b> \n If checked, cb2Bib elaborates the 'month'
    string on a BibTeX fashion. E.g., 'April 1' becomes '"1~" # apr'.

    - <b>Try Heuristic Guess if recognition fails</b> \n If checked, and the
    automatic recognition fails, cb2Bib tries to catch some of the fields of
    the reference through an heuristic set of rules. See \ref heuristic_guess.

    - <b>Page Number Separator</b> \n Sets separator, e.g., ' - ' or ' -- '.
    Spaces count.

    - <b>External BibTeX Postprocessing</b> \n Use this box to select a
    preferred BibTeX postprocessing external tool. The name of the executable,
    the command arguments and the output file extension are required fields.
    Arguments, any number, are passed to the executable. For the sake of
    generality, it is required to specify the <tt>\%finput</tt> and
    <tt>\%foutput</tt> placeholders. The first one is later substituted by the
    current BibTeX filename. The second one is substituted by the current
    filename with the specified output extension. <b>Caution:</b> Be careful if
    using the same file extension for input and output, e.g., using
    <tt>bib</tt> because you want to use a beautifier or a sorting script. The
    cb2Bib has no control on external disk modifications. Moreover, if the
    script fails, the input data might be lost. See also \ref keyboard.


\section c2bconf_pdf2text Configuring conversion to plain text

    - <b>To plain text converter</b> \n Selects the external
    <tt>some_format_to_text</tt> tool that cb2Bib uses to convert sets of files
    prior to reference extraction. cb2Bib executes the line command
    <tt>converter [options] inputfile tmp_output.txt</tt>, where
    <tt>[options]</tt> are user defined arguments. As a default, cb2Bib
    launches <tt>pdftotext</tt>, a PDF to text utility found in many platforms.
    Default arguments are <tt>-f 1 -l 1</tt> to extract only the first page
    when used within PDFImport, and <tt>-raw</tt>, to convert all the document,
    when used within Search in Files. Appropriate for PDFImport could also be a
    document metadata extractor. Often metadata contains structured information
    regarding document authors, title, and source.


    <p>&nbsp;</p>
*/

c2bConfigure::c2bConfigure(QWidget* parent) : QDialog(parent)
{
    ui.setupUi(this);
    settings = c2bSettingsP;

    ui.tabWidget->setCurrentIndex(settings->value("c2bConfigure/CurrentPage", 0).toInt());
    ui.buttonBox->button(QDialogButtonBox::Apply)->setAutoDefault(false);
    ui.buttonBox->button(QDialogButtonBox::Cancel)->setAutoDefault(false);
    ui.buttonBox->button(QDialogButtonBox::Help)->setAutoDefault(false);
    connect(ui.buttonBox->button(QDialogButtonBox::Apply), SIGNAL(clicked()), this, SLOT(apply()));
    connect(ui.buttonBox, SIGNAL(helpRequested()), this, SLOT(help()));

    // Files
    ui.JournalFile->setText(settings->fileName("cb2Bib/JournalFile"));
    connect(ui.JournalDir, SIGNAL(clicked()), this, SLOT(getJournalFile()));
    ui.RegularExpressionFile->setText(settings->fileName("cb2Bib/RegularExpressionFile"));
    connect(ui.RegularExpressionDir, SIGNAL(clicked()), this, SLOT(getRegularExpressionFile()));
    ui.NetworkFile->setText(settings->fileName("cb2Bib/NetworkFile"));
    connect(ui.NetworkDir, SIGNAL(clicked()), this, SLOT(getNetworkFile()));
    ui.BrowserCssFile->setText(settings->fileName("cb2Bib/BrowserCssFile"));
    connect(ui.BrowserCssDir, SIGNAL(clicked()), this, SLOT(getBrowserCssFile()));
    ui.CacheDirectory->setText(settings->fileName("cb2Bib/CacheDirectory"));
    connect(ui.CacheDir, SIGNAL(clicked()), this, SLOT(getCacheDirectory()));
    ui.MovePdf->setChecked(settings->value("cb2Bib/MovePdf").toBool());
    ui.PdfDirectory->setText(settings->fileName("cb2Bib/PdfDirectory"));
    connect(ui.PdfDir, SIGNAL(clicked()), this, SLOT(getPdfDirectory()));
    QString CopyMovePdf = settings->value("cb2Bib/CopyMovePdf").toString();
    if (CopyMovePdf == "copy")
        ui.Copy->setChecked(true);
    else if (CopyMovePdf == "move")
        ui.Move->setChecked(true);
    if (ui.MovePdf->isChecked())
        ui.RenameCopyMovePdf->setEnabled(true);
    else
        ui.RenameCopyMovePdf->setEnabled(false);
    connect(ui.MovePdf, SIGNAL(toggled(bool)), ui.RenameCopyMovePdf, SLOT(setEnabled(bool)));
    ui.RelativePdfDirectory->setChecked(settings->value("cb2Bib/RelativePdfDirectory").toBool());
    if (ui.RelativePdfDirectory->isChecked())
    {
        ui.PdfDirectory->setEnabled(false);
        ui.PdfDir->setEnabled(false);
    }
    connect(ui.RelativePdfDirectory, SIGNAL(toggled(bool)), ui.PdfDirectory, SLOT(setDisabled(bool)));
    connect(ui.RelativePdfDirectory, SIGNAL(toggled(bool)), ui.PdfDir, SLOT(setDisabled(bool)));

    // Fonts
    font = settings->c2bMonoFont;
    QFontDatabase db;
    ui.familycombo->insertItems(0, db.families());
    QList<int> sizes = db.standardSizes();
    QList<int>::Iterator it = sizes.begin();
    while (it != sizes.end())
        ui.psizecombo->insertItem(0, QString::number(*it++));
    FontSelection();
    connect(ui.familycombo, SIGNAL(activated(const QString&)), this, SLOT(comboFamilyChanged(const QString&)));
    connect(ui.psizecombo, SIGNAL(activated(const QString&)), this, SLOT(comboSizeChanged(const QString&)));
    colors.loadColors(settings);
    QHashIterator<QString, QColor*> cit(colors.colorList);
    while (cit.hasNext())
    {
        cit.next();
        QTreeWidgetItem* item = new QTreeWidgetItem(QStringList(cit.key()));
        item->setData(0, Qt::DecorationRole, *cit.value());
        item->setData(0, Qt::TextColorRole, *cit.value());
        ui.ColorList->insertTopLevelItem(0, item);
    }
    ui.ColorList->header()->setSortIndicatorShown(false);
    ui.ColorList->viewport()->setCursor(QCursor(static_cast<Qt::CursorShape>(Qt::PointingHandCursor)));
    ui.ColorList->sortItems(0, Qt::AscendingOrder);
    connect(ui.ColorList, SIGNAL(itemClicked(QTreeWidgetItem*, int)), this, SLOT(changeContextColor(QTreeWidgetItem*)));
    connect(ui.ColorList, SIGNAL(itemActivated(QTreeWidgetItem*, int)), this, SLOT(changeContextColor(QTreeWidgetItem*)));

    // Input
    ui.FindReplaceInput->setChecked(settings->value("cb2Bib/FindReplaceInput").toBool());
    connect(ui.FindReplaceInput, SIGNAL(toggled(bool)), this, SLOT(updateFindReplaceInputButtons()));
    updateFindReplaceInputButtons();
    QStringList FindReplaceInputList = settings->value("cb2Bib/FindReplaceInputList").toStringList();
    QStringList::Iterator i = FindReplaceInputList.begin();
    while (i != FindReplaceInputList.end())
    {
        QStringList spLine = (*i).split("|");
        new QTreeWidgetItem(ui.FindReplaceList, spLine);
        ++i;
    }
    connect(ui.FindReplaceList, SIGNAL(itemActivated(QTreeWidgetItem*, int)), this, SLOT(FindReplaceDialog_Edit()));
    connect(ui.FindReplaceList, SIGNAL(itemClicked(QTreeWidgetItem*, int)), this, SLOT(updateFindReplaceInputButtons()));
    connect(ui.FRNew, SIGNAL(clicked()), this, SLOT(FindReplaceDialog_New()));
    connect(ui.FREdit, SIGNAL(clicked()), this, SLOT(FindReplaceDialog_Edit()));
    connect(ui.FRDelete, SIGNAL(clicked()), this, SLOT(FindReplaceDialog_Delete()));
    ui.PreparserAutomatic->setChecked(settings->value("cb2Bib/PreparserAutomatic").toBool());
    ui.PreparserShowLog->setChecked(!settings->value("cb2Bib/PreparserShowLog").toBool());
    ui.PreparserBin->setText(settings->fileName("cb2Bib/PreparserBin"));
    connect(ui.PreparserDir, SIGNAL(clicked()), this, SLOT(getPreparserDir()));

    // Network
    ui.FmClient->setChecked(settings->value("cb2Bib/FmClient").toBool());
    ui.clientGroupBox->setEnabled(settings->value("cb2Bib/FmClient").toBool());
    ui.FmClientCopyBin->setText(settings->fileName("cb2Bib/FmClientCopyBin"));
    ui.FmClientCopyArg->setText(settings->value("cb2Bib/FmClientCopyArg").toString());
    ui.FmClientMoveBin->setText(settings->fileName("cb2Bib/FmClientMoveBin"));
    ui.FmClientMoveArg->setText(settings->value("cb2Bib/FmClientMoveArg").toString());
    ui.FmClientOpenBin->setText(settings->fileName("cb2Bib/FmClientOpenBin"));
    ui.FmClientOpenArg->setText(settings->value("cb2Bib/FmClientOpenArg").toString());
    connect(ui.FmClientCopyDir, SIGNAL(clicked()), this, SLOT(getFmClientCopyBin()));
    connect(ui.FmClientMoveDir, SIGNAL(clicked()), this, SLOT(getFmClientMoveBin()));
    connect(ui.FmClientOpenDir, SIGNAL(clicked()), this, SLOT(getFmClientOpenBin()));
    ui.AutomaticQuery->setChecked(settings->value("cb2Bib/AutomaticQuery").toBool());
    ui.AutomaticPdfDownload->setChecked(settings->value("cb2Bib/AutomaticPdfDownload").toBool());
    ui.KeepTmpNQFiles->setChecked(settings->value("cb2Bib/KeepTmpNQFiles").toBool());

    // Output
    ui.PostprocessMonths->setChecked(settings->value("cb2Bib/PostprocessMonths").toBool());
    ui.ConvertReferenceToLaTeX->setChecked(settings->value("cb2Bib/ConvertReferenceToLaTeX").toBool());
    ui.UseDoubleBraces->setChecked(settings->value("cb2Bib/UseDoubleBraces").toBool());
    ui.UseAbbreviatedNames->setChecked(!settings->value("cb2Bib/UseFullNames").toBool());
    ui.UseFullNames->setChecked(settings->value("cb2Bib/UseFullNames").toBool());
    ui.SetJournalsToAbbreviated->setChecked(!settings->value("cb2Bib/SetJournalsToFullname").toBool());
    ui.SetJournalsToFullname->setChecked(settings->value("cb2Bib/SetJournalsToFullname").toBool());
    ui.CiteIdPattern->setText(settings->value("cb2Bib/CiteIdPattern").toString());
    ui.DoHeuristicGuess->setChecked(settings->value("cb2Bib/DoHeuristicGuess").toBool());
    ui.PageNumberSeparator->setText(settings->value("cb2Bib/PageNumberSeparator").toString());
    ui.PostprocessBibTeXBin->setText(settings->fileName("cb2Bib/PostprocessBibTeXBin"));
    ui.PostprocessBibTeXArg->setText(settings->value("cb2Bib/PostprocessBibTeXArg").toString());
    ui.PostprocessBibTeXExt->setText(settings->value("cb2Bib/PostprocessBibTeXExt").toString());
    connect(ui.PostprocessBibTeXDir, SIGNAL(clicked()), this, SLOT(getPostprocessBibTeXDir()));

    // PDFImport
    ui.PdfImportPdf2TextBin->setText(settings->fileName("c2bPdfImport/Pdf2TextBin"));
    ui.PdfImportPdf2TextArg->setText(settings->value("c2bPdfImport/Pdf2TextArg").toString());
    connect(ui.PdfImportPdf2TextDir, SIGNAL(clicked()), this, SLOT(getPdfImportPdf2TextDir()));

    // Search
    ui.BibSearcherPdf2TextBin->setText(settings->fileName("c2bBibSearcher/Pdf2TextBin"));
    ui.BibSearcherPdf2TextArg->setText(settings->value("c2bBibSearcher/Pdf2TextArg").toString());
    connect(ui.BibSearcherPdf2TextDir, SIGNAL(clicked()), this, SLOT(getBibSearcherPdf2TextDir()));
}

c2bConfigure::~c2bConfigure()
{}


void c2bConfigure::accept()
{
    apply();
    QDialog::accept();
}

void c2bConfigure::apply()
{
    settings->setValue("c2bConfigure/CurrentPage", ui.tabWidget->currentIndex());

    // Files
    settings->setFileName("cb2Bib/JournalFile", ui.JournalFile->text());
    settings->setFileName("cb2Bib/RegularExpressionFile", ui.RegularExpressionFile->text());
    settings->setFileName("cb2Bib/NetworkFile", ui.NetworkFile->text());
    settings->setFileName("cb2Bib/BrowserCssFile", ui.BrowserCssFile->text());
    settings->setFileName("cb2Bib/CacheDirectory", ui.CacheDirectory->text());
    QString CopyMove;
    if (ui.Copy->isChecked())
        CopyMove = "copy";
    else
        CopyMove = "move";
    settings->setValue("cb2Bib/CopyMovePdf", CopyMove);
    settings->setValue("cb2Bib/MovePdf", ui.MovePdf->isChecked());
    settings->setFileName("cb2Bib/PdfDirectory", ui.PdfDirectory->text());
    settings->setValue("cb2Bib/RelativePdfDirectory", ui.RelativePdfDirectory->isChecked());

    // Fonts
    settings->setValue("cb2Bib/MonoFont", font);
    colors.saveColors(settings);

    // Input
    QStringList FindReplaceInputList;
    QTreeWidgetItemIterator it(ui.FindReplaceList);
    while (*it)
    {
        QTreeWidgetItem *item = *it;
        QString Line = QString("%1|%2|%3").arg(item->text(0)).arg(item->text(1)).arg(item->text(2));
        FindReplaceInputList.append(Line);
        ++it;
    }
    if (FindReplaceInputList.isEmpty())
    {
        FindReplaceInputList = C2B_FIND_REPLACE_LIST.split("^e");
        settings->setValue("cb2Bib/FindReplaceInput", false);
    }
    else
        settings->setValue("cb2Bib/FindReplaceInput", ui.FindReplaceInput->isChecked());
    settings->setValue("cb2Bib/FindReplaceInputList", FindReplaceInputList);
    settings->setValue("cb2Bib/PreparserAutomatic", ui.PreparserAutomatic->isChecked());
    settings->setValue("cb2Bib/PreparserShowLog", !ui.PreparserShowLog->isChecked());
    settings->setFileName("cb2Bib/PreparserBin", ui.PreparserBin->text());

    // Network
    settings->setValue("cb2Bib/FmClient", ui.FmClient->isChecked());
    settings->setFileName("cb2Bib/FmClientCopyBin", ui.FmClientCopyBin->text());
    settings->setFileName("cb2Bib/FmClientMoveBin", ui.FmClientMoveBin->text());
    settings->setFileName("cb2Bib/FmClientOpenBin", ui.FmClientOpenBin->text());
    settings->setValue("cb2Bib/FmClientCopyArg", ui.FmClientCopyArg->text().simplified());
    settings->setValue("cb2Bib/FmClientMoveArg", ui.FmClientMoveArg->text().simplified());
    settings->setValue("cb2Bib/FmClientOpenArg", ui.FmClientOpenArg->text().simplified());
    settings->setValue("cb2Bib/AutomaticQuery", ui.AutomaticQuery->isChecked());
    settings->setValue("cb2Bib/AutomaticPdfDownload", ui.AutomaticPdfDownload->isChecked());
    settings->setValue("cb2Bib/KeepTmpNQFiles", ui.KeepTmpNQFiles->isChecked());

    // Output
    settings->setValue("cb2Bib/PostprocessMonths", ui.PostprocessMonths->isChecked());
    settings->setValue("cb2Bib/ConvertReferenceToLaTeX", ui.ConvertReferenceToLaTeX->isChecked());
    settings->setValue("cb2Bib/UseDoubleBraces", ui.UseDoubleBraces->isChecked());
    settings->setValue("cb2Bib/UseFullNames", ui.UseFullNames->isChecked());
    settings->setValue("cb2Bib/SetJournalsToFullname", ui.SetJournalsToFullname->isChecked());
    settings->setFileName("cb2Bib/PostprocessBibTeXBin", ui.PostprocessBibTeXBin->text());
    settings->setValue("cb2Bib/PostprocessBibTeXArg", ui.PostprocessBibTeXArg->text());
    settings->setValue("cb2Bib/PostprocessBibTeXExt", ui.PostprocessBibTeXExt->text());
    settings->setValue("cb2Bib/DoHeuristicGuess", ui.DoHeuristicGuess->isChecked());
    settings->setValue("cb2Bib/PageNumberSeparator", ui.PageNumberSeparator->text());
    settings->setValue("cb2Bib/CiteIdPattern", ui.CiteIdPattern->text());

    // PDFImport
    settings->setValue("c2bPdfImport/Pdf2TextArg", ui.PdfImportPdf2TextArg->text().simplified());
    settings->setFileName("c2bPdfImport/Pdf2TextBin", ui.PdfImportPdf2TextBin->text());

    // Search
    settings->setValue("c2bBibSearcher/Pdf2TextArg", ui.BibSearcherPdf2TextArg->text().simplified());
    settings->setFileName("c2bBibSearcher/Pdf2TextBin", ui.BibSearcherPdf2TextBin->text());

    // Notify settings
    settings->notify();
}

void c2bConfigure::comboFamilyChanged(const QString& family)
{
    ui.samplelineedit->setFont(QFont(family, ui.psizecombo->currentText().toInt()));
    font.setFamily(family);
}

void c2bConfigure::comboSizeChanged(const QString& pt)
{
    ui.samplelineedit->setFont(QFont(ui.familycombo->currentText(), pt.toInt()));
    font.setPointSize(pt.toInt());
}

void c2bConfigure::FindReplaceDialog_Delete()
{
    QTreeWidgetItem* item = ui.FindReplaceList->currentItem();
    if (item)
        delete item;
    item = ui.FindReplaceList->currentItem();
    if (item)
        item->setSelected(true);
    updateFindReplaceInputButtons();
}

void c2bConfigure::FindReplaceDialog_Edit()
{
    QTreeWidgetItem* item = ui.FindReplaceList->currentItem();
    if (item)
    {
        c2bConfigure_FR* frD = new c2bConfigure_FR(item, this);
        frD->exec();
        delete frD;
    }
}

void c2bConfigure::FindReplaceDialog_New()
{
    QStringList newlist;
    newlist << "" << "" << "";
    QTreeWidgetItem *fri = new QTreeWidgetItem(ui.FindReplaceList, newlist);
    c2bConfigure_FR* frD = new c2bConfigure_FR(fri, this);
    if (frD->exec() != QDialog::Accepted)
        delete fri;
    delete frD;
}

void c2bConfigure::updateFindReplaceInputButtons()
{
    if (!ui.FindReplaceInput->isChecked())
    {
        ui.FindReplaceList->setEnabled(false);
        ui.FRNew->setEnabled(false);
        ui.FREdit->setEnabled(false);
        ui.FRDelete->setEnabled(false);
        return;
    }
    else
    {
        ui.FindReplaceList->setEnabled(true);
        ui.FRNew->setEnabled(true);
        if (ui.FindReplaceList->topLevelItemCount() > 0)
        {
            ui.FREdit->setEnabled(true);
            ui.FRDelete->setEnabled(true);
        }
        else
        {
            ui.FREdit->setEnabled(false);
            ui.FRDelete->setEnabled(false);
        }
    }
}

void c2bConfigure::getJournalFile()
{
    QString fn = ui.JournalFile->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, tr("Select a filename (Restart to apply changes)"),
                     fn, tr("cb2Bib Journal Lists (*.txt);;All (*)"));
    if (!new_fn.isEmpty())
        ui.JournalFile->setText(new_fn);
}

void c2bConfigure::getRegularExpressionFile()
{
    QString fn = ui.RegularExpressionFile->text();
    QString new_fn = c2bFileDialog::getFileName(this, "", fn, tr("cb2Bib RegExp Lists (*.txt);;All (*)"));
    if (!new_fn.isEmpty())
        ui.RegularExpressionFile->setText(new_fn);
}

void c2bConfigure::getNetworkFile()
{
    QString fn = ui.NetworkFile->text();
    QString new_fn = c2bFileDialog::getFileName(this, "", fn, tr("cb2Bib Network Query Info (*.txt);;All (*)"));
    if (!new_fn.isEmpty())
        ui.NetworkFile->setText(new_fn);
}

void c2bConfigure::getBrowserCssFile()
{
    QString fn = ui.BrowserCssFile->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, tr("Select a filename (Restart editor to apply changes)"),
                     fn, tr("Cascading Style Sheets (*.css);;All (*)"));
    if (!new_fn.isEmpty())
        ui.BrowserCssFile->setText(new_fn);
}

void c2bConfigure::getFmClientCopyBin()
{
    QString fn = ui.FmClientCopyBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("File Manager Client executable (*)"));
    if (!new_fn.isEmpty())
        ui.FmClientCopyBin->setText(new_fn);
}

void c2bConfigure::getPreparserDir()
{
    QString fn = ui.PreparserBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("External Parser executable (*)"));
    if (!new_fn.isEmpty())
        ui.PreparserBin->setText(new_fn);
}

void c2bConfigure::getFmClientMoveBin()
{
    QString fn = ui.FmClientMoveBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("File Manager Client executable (*)"));
    if (!new_fn.isEmpty())
        ui.FmClientMoveBin->setText(new_fn);
}

void c2bConfigure::getFmClientOpenBin()
{
    QString fn = ui.FmClientOpenBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("File Manager Client executable (*)"));
    if (!new_fn.isEmpty())
        ui.FmClientOpenBin->setText(new_fn);
}

void c2bConfigure::getPostprocessBibTeXDir()
{
    QString fn = ui.PostprocessBibTeXBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("BibTeX Postprocess executable (*)"));
    if (!new_fn.isEmpty())
        ui.PostprocessBibTeXBin->setText(new_fn);
}

void c2bConfigure::getPdfImportPdf2TextDir()
{
    QString fn = ui.PdfImportPdf2TextBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("PDF to text converter (*)"));
    if (!new_fn.isEmpty())
        ui.PdfImportPdf2TextBin->setText(new_fn);
}

void c2bConfigure::getBibSearcherPdf2TextDir()
{
    QString fn = ui.BibSearcherPdf2TextBin->text();
    QString new_fn = c2bFileDialog::getOpenFileName(this, "", fn, tr("PDF to text converter (*)"));
    if (!new_fn.isEmpty())
        ui.BibSearcherPdf2TextBin->setText(new_fn);
}

void c2bConfigure::getCacheDirectory()
{
    QString dir = ui.CacheDirectory->text();
    if (dir.isEmpty())
        dir = QDir::homePath();
    QString new_dir = c2bFileDialog::getExistingDirectory(this, tr("Select a cache directory"), dir);
    if (!new_dir.isEmpty())
        ui.CacheDirectory->setText(new_dir);
}

void c2bConfigure::getPdfDirectory()
{
    QString dir = ui.PdfDirectory->text();
    if (dir.isEmpty())
        dir = QDir::homePath();
    QString new_dir = c2bFileDialog::getExistingDirectory(this, tr("Select a PDF directory"), dir);
    if (!new_dir.isEmpty())
        ui.PdfDirectory->setText(new_dir);
}

void c2bConfigure::FontSelection()
{
    ui.familycombo->setCurrentFont(font);
    int p_font_size = font.pointSize();
    int i = 0;
    while (i < ui.psizecombo->count())
    {
        if (ui.psizecombo->itemText(i) == QString::number(p_font_size))
        {
            ui.psizecombo->setCurrentIndex(i);
            break;
        }
        i++;
    }
    ui.samplelineedit->setFont(font);
}

void c2bConfigure::changeContextColor(QTreeWidgetItem* item)
{
    QColor newColor = QColorDialog::getColor(item->textColor(0), this);
    if (newColor.isValid())
    {
        item->setData(0, Qt::TextColorRole, newColor);
        item->setData(0, Qt::DecorationRole, newColor);
        colors.updateColor(newColor, item->text(0));
    }
}

void c2bConfigure::help()
{
    c2b::displayHelp("http://www.molspaces.com/d_cb2bib-c2bconf.php");
}
