/***************************************************************************
 *   Copyright (C) 2004-2008 by Pere Constans
 *   constans@molspaces.com
 *   cb2Bib version 0.9.4
 *   See the LICENSE file that comes with this distribution
 ***************************************************************************/
#ifndef C2BUTILS_H
#define C2BUTILS_H

#include <QFile>
#include <QGraphicsTextItem>
#include <QRegExp>
#include <QString>
#include <QtDebug>


/*!
    General cb2Bib Utils

    @author Pere Constans
*/

namespace c2bUtils
{

extern QString c2bToAscii(QString str, bool keepWords = false);
extern QString& bibToC2b(QString& str);
extern QString& c2bToBib(QString& str);

const QString PNAS_greek_letters = "[\\{\\[\\(](alpha|beta|gamma|delta|varepsilon|"
                                   "zeta|eta|theta|iota|kappa|lambda|mu|nu|xi|"
                                   "omicron|pi|rho|sigmaf|varsigma|sigma|tau|"
                                   "upsilon|phi|chi|psi|omega|vartheta|varphi|"
                                   "varpi|digamma|varkappa|varrho|epsilon)[\\}\\]\\)]";
static const QRegExp isBibToC2b("[\\{\\[\\$\\\\]");
static const QRegExp isC2bToBib("[^A-Za-z\\.,;\\s\\-]");

inline QString& fullBibToC2b(QString& str)
{
    // Escape TeX and other special characters to Unicode
    if (!str.contains(isBibToC2b))
        return str;
    // PNAS Style
    str.replace(QRegExp(PNAS_greek_letters, Qt::CaseInsensitive), "$\\\\1$");
    str.replace(QRegExp("\\{middle dot\\}"), "$\\cdot$");
    bibToC2b(str);
    return str;
}

inline QString fileToString(const QString& fn, bool delete_on_close = false)
{
    if (fn.isEmpty())
        return QString();
    QFile f(fn);
    if (f.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream stream(&f);
        QString contents = stream.readAll();
        if (delete_on_close)
            f.remove();
        else
            f.close();
        return (contents);
    }
    else
        return QString();
}

inline QString firstPage(QString pages)
{
    pages.simplified();
    pages.replace(QRegExp("[\\s-].*$"), "");
    return (pages);
}

inline QString fromHtmlString(const QString& str)
{
    QGraphicsTextItem converter;
    converter.setHtml(str);
    QString converted = converter.toPlainText();
    return (converted);
}

inline QString setCapitalization(const QString& str)
{
    QString cap_string = str;
    cap_string = cap_string.toLower();
    bool do_upper = true;
    for (int i = 0; i < cap_string.length(); i++)
        if (cap_string[i].isLetter())
        {
            if (do_upper)
                cap_string[i] = cap_string[i].toUpper();
            do_upper = false;
        }
        else if (cap_string[i] == '.' || cap_string[i] == ':')
            do_upper = true;
    return (cap_string);
}

inline QString toHtmlString(QString str)
{
    // Encode some symbols to Html for proper browser display
    str.replace("<", "&#060;");
    str.replace(">", "&#062;");
    str.replace("%", "&#037;");
    return (str);
}

inline bool isUpperCaseString(const QString& str)
{
    if (str.isEmpty())
        return false;
    for (int i = 0; i < str.length(); i++)
        if (str[i].isLetter())
            if (str[i].category() == QChar::Letter_Lowercase)
                return false;
    return true;
}

inline int nearInteger(double a)
{
    int ia = (int)a;
    a = a - (double)ia;
    if (a >= 0.5)
        ia++;
    else if (a <= -0.5)
        ia--;
    return ia;
}

inline void debug(const QString&
#ifdef CB2BIB_DEBUG
                  debug_ms
#endif
                 )
{
#ifdef CB2BIB_DEBUG
    qDebug() << "[cb2bib] " << debug_ms;
#endif
}

} // namespace c2bUtils

#endif
